<?php

/**
 * @file
 * Database to mimic the installation of the VG Wort module.
 */

use Drupal\Core\Database\Database;
use Drupal\Core\Serialization\Yaml;
use Drupal\vgwort\EntityJobMapper;

include_once __DIR__ . '/../../vgwort.install';

$connection = Database::getConnection();

// Set the schema version.
$connection->merge('key_value')
  ->condition('collection', 'system.schema')
  ->condition('name', 'vgwort')
  ->fields([
    'collection' => 'system.schema',
    'name' => 'vgwort',
    'value' => 'i:20003;',
  ])
  ->execute();

// Update core.extension.
$extensions = $connection->select('config')
  ->fields('config', ['data'])
  ->condition('collection', '')
  ->condition('name', 'core.extension')
  ->execute()
  ->fetchField();
$extensions = unserialize($extensions);
$extensions['module']['advancedqueue'] = 0;
$extensions['module']['vgwort'] = 0;
$connection->update('config')
  ->fields([
    'data' => serialize($extensions),
  ])
  ->condition('collection', '')
  ->condition('name', 'core.extension')
  ->execute();

// Install VG Wort settings so tht entities can get a counter ID.
$config = <<<YAML
username: ''
password: ''
prefix: vgzm
publisher_id: '123456'
image_domain: 'http://example.com'
registration_wait_days: 14
queue_retry_time: 86400
entity_types:
  node:
    view_mode: full
test_mode: false
legal_rights:
  distribution: false
  public_access: false
  reproduction: false
  declaration_of_granting: false
  other_public_communication: false
YAML;

$connection->insert('config')
  ->fields([
    'collection',
    'name',
    'data',
  ])
  ->values([
    'collection' => '',
    'name' => 'vgwort.settings',
    'data' => serialize(Yaml::decode($config)),
  ])
  ->execute();

// Install VG Wort queue are they were for the third beta release.
$config = <<<YAML
uuid: 34e0cd0a-8230-41d2-a724-0717ab69996f
langcode: en
status: true
dependencies:
  enforced:
    module:
      - vgwort
id: vgwort
label: 'VG Wort'
backend: database
backend_configuration:
  lease_time: 300
processor: daemon
processing_time: 90
locked: false
threshold:
  type: 0
  limit: 0
  state: all
YAML;

$connection->insert('config')
  ->fields([
    'collection',
    'name',
    'data',
  ])
  ->values([
    'collection' => '',
    'name' => 'advancedqueue.advancedqueue_queue.vgwort',
    'data' => serialize(Yaml::decode($config)),
  ])
  ->execute();

$entity_type = "O:42:\"Drupal\\Core\\Config\\Entity\\ConfigEntityType\":43:{s:5:\"\0*\0id\";s:19:\"advancedqueue_queue\";s:8:\"\0*\0class\";s:33:\"Drupal\\advancedqueue\\Entity\\Queue\";s:11:\"\0*\0provider\";s:13:\"advancedqueue\";s:15:\"\0*\0static_cache\";b:0;s:15:\"\0*\0render_cache\";b:1;s:19:\"\0*\0persistent_cache\";b:1;s:14:\"\0*\0entity_keys\";a:8:{s:2:\"id\";s:2:\"id\";s:5:\"label\";s:5:\"label\";s:4:\"uuid\";s:4:\"uuid\";s:8:\"revision\";s:0:\"\";s:6:\"bundle\";s:0:\"\";s:8:\"langcode\";s:8:\"langcode\";s:16:\"default_langcode\";s:16:\"default_langcode\";s:29:\"revision_translation_affected\";s:29:\"revision_translation_affected\";}s:16:\"\0*\0originalClass\";s:33:\"Drupal\\advancedqueue\\Entity\\Queue\";s:11:\"\0*\0handlers\";a:5:{s:6:\"access\";s:46:\"Drupal\\advancedqueue\\QueueAccessControlHandler\";s:12:\"list_builder\";s:37:\"Drupal\\advancedqueue\\QueueListBuilder\";s:4:\"form\";a:3:{s:3:\"add\";s:35:\"Drupal\\advancedqueue\\Form\\QueueForm\";s:4:\"edit\";s:35:\"Drupal\\advancedqueue\\Form\\QueueForm\";s:6:\"delete\";s:35:\"Drupal\\Core\\Entity\\EntityDeleteForm\";}s:14:\"route_provider\";a:1:{s:7:\"default\";s:51:\"Drupal\\Core\\Entity\\Routing\\DefaultHtmlRouteProvider\";}s:7:\"storage\";s:45:\"Drupal\\Core\\Config\\Entity\\ConfigEntityStorage\";}s:19:\"\0*\0admin_permission\";s:24:\"administer advancedqueue\";s:25:\"\0*\0permission_granularity\";s:11:\"entity_type\";s:8:\"\0*\0links\";a:4:{s:8:\"add-form\";s:31:\"/admin/config/system/queues/add\";s:9:\"edit-form\";s:56:\"/admin/config/system/queues/manage/{advancedqueue_queue}\";s:11:\"delete-form\";s:63:\"/admin/config/system/queues/manage/{advancedqueue_queue}/delete\";s:10:\"collection\";s:27:\"/admin/config/system/queues\";}s:21:\"\0*\0bundle_entity_type\";N;s:12:\"\0*\0bundle_of\";N;s:15:\"\0*\0bundle_label\";N;s:13:\"\0*\0base_table\";N;s:22:\"\0*\0revision_data_table\";N;s:17:\"\0*\0revision_table\";N;s:13:\"\0*\0data_table\";N;s:11:\"\0*\0internal\";b:0;s:15:\"\0*\0translatable\";b:0;s:19:\"\0*\0show_revision_ui\";b:0;s:8:\"\0*\0label\";O:48:\"Drupal\\Core\\StringTranslation\\TranslatableMarkup\":3:{s:9:\"\0*\0string\";s:5:\"Queue\";s:12:\"\0*\0arguments\";a:0:{}s:10:\"\0*\0options\";a:0:{}}s:19:\"\0*\0label_collection\";O:48:\"Drupal\\Core\\StringTranslation\\TranslatableMarkup\":3:{s:9:\"\0*\0string\";s:6:\"Queues\";s:12:\"\0*\0arguments\";a:0:{}s:10:\"\0*\0options\";a:0:{}}s:17:\"\0*\0label_singular\";O:48:\"Drupal\\Core\\StringTranslation\\TranslatableMarkup\":3:{s:9:\"\0*\0string\";s:5:\"queue\";s:12:\"\0*\0arguments\";a:0:{}s:10:\"\0*\0options\";a:0:{}}s:15:\"\0*\0label_plural\";O:48:\"Drupal\\Core\\StringTranslation\\TranslatableMarkup\":3:{s:9:\"\0*\0string\";s:6:\"queues\";s:12:\"\0*\0arguments\";a:0:{}s:10:\"\0*\0options\";a:0:{}}s:14:\"\0*\0label_count\";a:3:{s:8:\"singular\";s:12:\"@count queue\";s:6:\"plural\";s:13:\"@count queues\";s:7:\"context\";N;}s:15:\"\0*\0uri_callback\";N;s:8:\"\0*\0group\";s:13:\"configuration\";s:14:\"\0*\0group_label\";O:48:\"Drupal\\Core\\StringTranslation\\TranslatableMarkup\":3:{s:9:\"\0*\0string\";s:13:\"Configuration\";s:12:\"\0*\0arguments\";a:0:{}s:10:\"\0*\0options\";a:1:{s:7:\"context\";s:17:\"Entity type group\";}}s:22:\"\0*\0field_ui_base_route\";N;s:26:\"\0*\0common_reference_target\";b:0;s:22:\"\0*\0list_cache_contexts\";a:0:{}s:18:\"\0*\0list_cache_tags\";a:1:{i:0;s:31:\"config:advancedqueue_queue_list\";}s:14:\"\0*\0constraints\";a:0:{}s:13:\"\0*\0additional\";a:0:{}s:14:\"\0*\0_serviceIds\";a:0:{}s:18:\"\0*\0_entityStorages\";a:0:{}s:20:\"\0*\0stringTranslation\";N;s:16:\"\0*\0config_prefix\";s:19:\"advancedqueue_queue\";s:14:\"\0*\0lookup_keys\";a:1:{i:0;s:4:\"uuid\";}s:16:\"\0*\0config_export\";a:8:{i:0;s:2:\"id\";i:1;s:5:\"label\";i:2;s:7:\"backend\";i:3;s:21:\"backend_configuration\";i:4;s:9:\"processor\";i:5;s:15:\"processing_time\";i:6;s:9:\"threshold\";i:7;s:6:\"locked\";}s:21:\"\0*\0mergedConfigExport\";a:0:{}}";
$connection->insert('key_value')
  ->fields([
    'collection',
    'name',
    'value',
  ])
  ->values([
    'collection' => 'entity.definitions.installed',
    'name' => 'advancedqueue_queue.entity_type',
    'value' => $entity_type,
  ])
  ->execute();


// Create the vgwort_entity_registration table.
$connection->schema()->createTable(EntityJobMapper::TABLE, _vgwort_schema_definition_20001());
